"""Helpers for Roku Client."""
from __future__ import annotations

import mimetypes
from ipaddress import ip_address
from socket import gaierror as SocketGIAError

import yarl

from .exceptions import RokuConnectionError
from .resolver import ThreadedResolver

MIME_TO_STREAM_FORMAT = {
    "application/dash+xml": "dash",
    "application/x-mpegURL": "hls",
    "application/vnd.apple.mpegurl": "hls",
    "audio/mpeg": "mp3",
    "audio/x-ms-wma": "wma",
    "video/mp4": "mp4",
    "video/quicktime": "mp4",
    "video/x-matroska": "mkv",
}


def guess_stream_format(  # pylint: disable=too-many-return-statements
    url: str, mime_type: str | None = None
) -> str | None:
    """Guess the Roku stream format for a given URL and MIME type.

    Args:
        url: The URL to determine stream format for.
        mime_type: The MIME type to aid in stream format determination.

    Returns:
        The stream format or None if unable to determine stream format.
    """
    parsed = yarl.URL(url)
    parsed_name = parsed.name.lower()

    if mime_type is None:
        mime_type, _ = mimetypes.guess_type(parsed.path)

    if mime_type == "audio/mpeg" and parsed_name.endswith(".m4a"):
        return "m4a"

    if mime_type is None:
        if parsed_name.endswith(".dash"):
            return "dash"
        if parsed_name.endswith(".mpd"):
            return "dash"
        if parsed_name.endswith(".m4v"):
            return "mp4"
        if parsed_name.endswith(".mks"):
            return "mks"
        if parsed_name.endswith(".mka"):
            return "mka"
        if ".ism/manifest" in parsed.path.lower():
            return "ism"

    if mime_type is None:
        return None

    if mime_type not in MIME_TO_STREAM_FORMAT:
        return None

    return MIME_TO_STREAM_FORMAT[mime_type]


def is_ip_address(host: str) -> bool:
    """Determine if host is an IP Address.

    Args:
        host: The hostname to check.

    Returns:
        Whether the provided hostname is an IP address.
    """
    try:
        ip_address(host)
    except ValueError:
        return False

    return True


async def resolve_hostname(host: str) -> str:
    """Resolve hostname to IP Address (asynchronously).

    Args:
        host: The hostname to resolve.

    Returns:
        The resolved IP address.

    Raises:
        RokuConnectionError: An error occurred while communicating with
            the Roku device.
    """
    try:
        resolver = ThreadedResolver()
        results = await resolver.resolve(host)
        ips = [ip_address(x["host"]) for x in results]
        return str(ips[0])
    except (OSError, SocketGIAError, ValueError) as exception:
        raise RokuConnectionError(
            f"Error occurred while resolving hostname: {host}"
        ) from exception



"""Constants for Roku."""

VALID_REMOTE_KEYS = {
    # Standard Keys
    "home": "Home",
    "reverse": "Rev",
    "forward": "Fwd",
    "play": "Play",
    "select": "Select",
    "left": "Left",
    "right": "Right",
    "down": "Down",
    "up": "Up",
    "back": "Back",
    "replay": "InstantReplay",
    "info": "Info",
    "backspace": "Backspace",
    "search": "Search",
    "enter": "Enter",
    # For devices that support "Find Remote"
    "find_remote": "FindRemote",
    # For Roku TV
    "volume_down": "VolumeDown",
    "volume_up": "VolumeUp",
    "volume_mute": "VolumeMute",
    # For Roku TV while on TV tuner channel
    "channel_up": "ChannelUp",
    "channel_down": "ChannelDown",
    # For Roku TV current input
    "input_tuner": "InputTuner",
    "input_hdmi1": "InputHDMI1",
    "input_hdmi2": "InputHDMI2",
    "input_hdmi3": "InputHDMI3",
    "input_hdmi4": "InputHDMI4",
    "input_av1": "InputAV1",
    # For devices that support being turned on/off
    "power": "Power",
    "poweroff": "PowerOff",
    "poweron": "PowerOn",
}



"""Models for Roku."""
from __future__ import annotations

from dataclasses import asdict, dataclass
from datetime import datetime
from math import floor
from typing import Any

from .exceptions import RokuError


def _ms_to_sec(msec: str) -> int:
    """Convert millisecond string to seconds integer.

    Args:
        msec: The number of milliseconds as a string.

    Returns:
        The number of seconds converted from milliseconds.
    """
    msi = int(msec.replace("ms", "").strip())
    return floor(msi / 1000)


@dataclass
class Application:
    """Object holding application information from Roku."""

    app_id: str | None
    name: str | None
    version: str | None
    screensaver: bool

    @staticmethod
    def from_dict(data: dict[str, Any]) -> Application:
        """Return Application object from Roku API response.

        Args:
            data: Dictionary of data.

        Returns:
            The Application object.
        """
        app = data.get("app", data)

        if isinstance(app, str):
            app = {"#text": app}

        return Application(
            app_id=app.get("@id", None),
            name=app.get("#text", None),
            version=app.get("@version", None),
            screensaver=data.get("screensaver") is not None,
        )


@dataclass
class Info:
    """Object holding device information from Roku."""

    name: str | None
    brand: str
    device_type: str
    device_location: str | None
    model_name: str | None
    model_number: str | None
    serial_number: str | None
    version: str | None
    network_type: str | None = None
    network_name: str | None = None
    ethernet_support: bool | None = None
    ethernet_mac: str | None = None
    wifi_mac: str | None = None
    supports_airplay: bool | None = None
    supports_find_remote: bool | None = None
    supports_private_listening: bool | None = None
    headphones_connected: bool | None = None

    @staticmethod
    def from_dict(data: dict[str, Any]) -> Info:
        """Return Info object from Roku API response.

        Args:
            data: Dictionary of data.

        Returns:
            The Info object.
        """
        device_type = "box"

        if data.get("is-tv", "false") == "true":
            device_type = "tv"
        elif data.get("is-stick", "false") == "true":
            device_type = "stick"

        airplay = data.get("supports-airplay", "false") == "true"
        find_remote = data.get("supports-find-remote", "false") == "true"
        private_listening = data.get("supports-private-listening", "false") == "true"

        return Info(
            name=data.get("user-device-name", None),
            brand=data.get("vendor-name", "Roku"),
            device_type=device_type,
            device_location=data.get("user-device-location", None),
            model_name=data.get("model-name", None),
            model_number=data.get("model-number", None),
            network_type=data.get("network-type", None),
            network_name=data.get("network-name", None),
            serial_number=data.get("serial-number", None),
            version=data.get("software-version", None),
            ethernet_support=data.get("supports-ethernet", "false") == "true",
            ethernet_mac=data.get("ethernet-mac", None),
            wifi_mac=data.get("wifi-mac", None),
            supports_airplay=airplay,
            supports_find_remote=find_remote,
            supports_private_listening=private_listening,
            headphones_connected=data.get("headphones-connected", "false") == "true",
        )


@dataclass
class Channel:
    """Object holding all information of TV Channel."""

    name: str | None
    number: str
    channel_type: str
    hidden: bool
    program_title: str | None = None
    program_description: str | None = None
    program_rating: str | None = None
    signal_mode: str | None = None
    signal_strength: int | None = None

    @staticmethod
    def from_dict(data: dict) -> Channel:
        """Return Channel object from Roku response.

        Args:
            data: Dictionary of data.

        Returns:
            The Channel object.
        """
        if (strength := data.get("signal-strength", None)) is not None:
            try:
                strength = int(strength)
            except ValueError:
                strength = None

        return Channel(
            name=data.get("name", None),
            number=data.get("number", "0"),
            channel_type=data.get("type", "unknown"),
            hidden=data.get("user-hidden", "false") == "true",
            program_title=data.get("program-title", None),
            program_description=data.get("program-description", None),
            program_rating=data.get("program-ratings", None),
            signal_mode=data.get("signal-mode", None),
            signal_strength=strength,
        )


@dataclass
class MediaState:
    """Object holding all information of media state."""

    duration: int
    live: bool
    paused: bool
    position: int
    at: datetime = datetime.utcnow()  # pylint: disable=C0103

    @staticmethod
    def from_dict(data: dict) -> MediaState | None:
        """Return MediaStste object from Roku response.

        Args:
            data: Dictionary of data.

        Returns:
            The MediaState object.
        """
        if (state := data.get("@state", None)) not in ("play", "pause"):
            return None

        duration = data.get("duration", "0")
        position = data.get("position", "0")

        return MediaState(
            live=data.get("is_live", "false") == "true",
            paused=state == "pause",
            duration=_ms_to_sec(duration),
            position=_ms_to_sec(position),
        )


@dataclass
class State:
    """Object holding all information of device state."""

    available: bool
    standby: bool
    at: datetime = datetime.utcnow()  # pylint: disable=C0103


class Device:
    """Object holding all information of device."""

    info: Info
    state: State
    apps: list[Application] = []
    channels: list[Channel] = []
    app: Application | None = None
    channel: Channel | None = None
    media: MediaState | None = None

    def __init__(self, data: dict[str, Any]):
        """Initialize an empty Roku device class.

        Args:
            data: Dictionary of data.

        Raises:
            RokuError: Received an unexpected response from the Roku device.
        """
        # Check if all elements are in the passed dict, else raise an Error
        if any(k not in data for k in ("info", "available", "standby")):
            raise RokuError("Roku data is incomplete, cannot construct device object")

        self.update_from_dict(data)

    def as_dict(self) -> dict[str, Any]:
        """Return dictionary version of the Roku device.

        Returns:
            A Python dictionary created from the Device object attributes.
        """
        apps = None
        if self.apps is not None:
            apps = [asdict(app) for app in self.apps]

        channels = None
        if self.channels is not None:
            channels = [asdict(channel) for channel in self.channels]

        app = None
        if self.app is not None:
            app = asdict(self.app)

        channel = None
        if self.channel is not None:
            channel = asdict(self.channel)

        media = None
        if self.media is not None:
            media = asdict(self.media)

        return {
            "info": asdict(self.info),
            "state": asdict(self.state),
            "apps": apps,
            "channels": channels,
            "app": app,
            "channel": channel,
            "media": media,
        }

    def update_from_dict(
        self, data: dict[str, Any], update_state: bool = True
    ) -> Device:
        """Return Device object from Roku device data.

        Args:
            data: Dictionary of data.
            update_state: Whether to update state attributes.

        Returns:
            The Device object.
        """
        if update_state:
            self.state = State(
                available=data.get("available", False),
                standby=data.get("standby", False),
            )

        if "info" in data and data["info"]:
            self.info = Info.from_dict(data["info"])

        if "apps" in data and data["apps"]:
            self.apps = [
                Application.from_dict(app_data)
                for app_data in data["apps"]
                if data["apps"] is not None
            ]

        if "channels" in data and data["channels"]:
            self.channels = [
                Channel.from_dict(channel_data)
                for channel_data in data["channels"]
                if data["channels"] is not None
            ]

        if "app" in data and data["app"]:
            self.app = Application.from_dict(data["app"])
        elif "app" in data:
            self.app = None

        if "channel" in data and data["channel"]:
            self.channel = Channel.from_dict(data["channel"])
        elif "channel" in data:
            self.channel = None

        if "media" in data and data["media"]:
            self.media = MediaState.from_dict(data["media"])
        elif "media" in data:
            self.media = None

        return self



"""Asynchronous Python client for Roku."""
from __future__ import annotations

import asyncio
import logging
from collections import OrderedDict
from dataclasses import dataclass
from importlib import metadata
from socket import gaierror as SocketGIAError
from typing import Any
from urllib.parse import quote_plus, urlencode
from xml.parsers.expat import ExpatError

import async_timeout
import xmltodict
from aiohttp.client import ClientError, ClientSession
from cachetools import TTLCache
from yarl import URL

from .const import VALID_REMOTE_KEYS
from .exceptions import RokuConnectionError, RokuError
from .helpers import is_ip_address, resolve_hostname
from .models import Device

LOGGER = logging.getLogger(__package__)


@dataclass
class Roku:
    """Main class for Python API."""

    host: str
    base_path: str = "/"
    port: int = 8060
    request_timeout: int = 5
    session: ClientSession | None = None
    user_agent: str | None = None

    _close_session: bool = False
    _dns_lookup: bool = False
    _dns_cache: TTLCache = TTLCache(maxsize=16, ttl=7200)
    _device: Device | None = None
    _scheme: str = "http"

    def __post_init__(self):
        """Initialize connection parameters."""
        if not is_ip_address(self.host):
            self._dns_lookup = True

        if self.user_agent is None:
            version = metadata.version(__package__)

            self.user_agent = f"PythonRokuECP/{version}"

    async def _request(
        self,
        uri: str = "",
        method: str = "GET",
        data: Any | None = None,
        params: dict[str, Any] | None = None,
        encoded: bool = False,
    ) -> Any:
        """Handle a request to a Roku device.

        Args:
            uri: Request URI, for example `/query/device-info`.
            method: HTTP method to use for the request.E.g., "GET" or "POST".
            data: Dictionary of data to send to the Roku device.
            params: Dictionary of request parameters to send to the Roku device.
            encoded: Whether the URI has already been url encoded.

        Returns:
            A Python dictionary (XML decoded) with the response from the
            Roku device.

        Raises:
            RokuConnectionError: An error occurred while communicating with
                the Roku device.
            RokuError: Received an unexpected response from the Roku device.
        """
        host = self.host

        if self._dns_lookup:
            try:
                host = self._dns_cache["ip_address"]
            except KeyError:
                host = await resolve_hostname(self.host)
                self._dns_cache["ip_address"] = host

        url = URL.build(
            scheme=self._scheme,
            host=host,
            port=self.port,
            path=self.base_path,
        ).join(URL(uri, encoded=encoded))

        headers = {
            "User-Agent": self.user_agent,
            "Accept": "application/xml, text/xml, text/plain, */*",
        }

        if self.session is None:
            self.session = ClientSession()
            self._close_session = True

        try:
            async with async_timeout.timeout(self.request_timeout):
                response = await self.session.request(
                    method,
                    url,
                    data=data,
                    params=params,
                    headers=headers,
                )
        except asyncio.TimeoutError as exception:
            raise RokuConnectionError(
                "Timeout occurred while connecting to device"
            ) from exception
        except (ClientError, SocketGIAError) as exception:
            raise RokuConnectionError(
                "Error occurred while communicating with device"
            ) from exception

        content_type = response.headers.get("Content-Type", "")

        if (response.status // 100) in [4, 5]:
            content = await response.read()
            response.close()

            raise RokuError(
                f"HTTP {response.status}",
                {
                    "content-type": content_type,
                    "message": content.decode("utf8"),
                    "status-code": response.status,
                },
            )

        if "application/xml" in content_type or "text/xml" in content_type:
            content = await response.text()

            try:
                data = xmltodict.parse(content)
                LOGGER.debug("Requesting %s returned %s", url, data)
            except (ExpatError, IndexError) as error:
                raise RokuError from error

            return data

        return await response.text()

    @property
    def device(self) -> Device | None:
        """Get Roku device information.

        Returns:
            A Device object with information about the Roku device.
        """
        return self._device

    def app_icon_url(self, app_id: str) -> str:
        """Get the URL to the application icon.

        Args:
            app_id: The application ID.

        Returns:
            The URL to the icon for the requested application ID.
        """
        icon_url = URL.build(
            scheme=self._scheme, host=self.host, port=self.port, path=self.base_path
        ).join(URL(f"query/icon/{app_id}"))

        return str(icon_url)

    async def update(  # pylint: disable=too-many-branches
        self, full_update: bool = False
    ) -> Device:
        """Get all information about the device in a single call.

        Args:
            full_update: Should application and channel lists be updated.

        Returns:
            A Device object, with information about the Roku device.
        """
        if self._device is None:
            full_update = True

        updates: dict = {}
        updates["info"] = None
        updates["available"] = True
        updates["standby"] = False
        updates["app"] = None
        updates["channel"] = None
        updates["media"] = None

        if full_update:
            updates["apps"] = []
            updates["channels"] = []

        updates["info"] = info = await self._get_device_info()

        if info.get("power-mode") != "PowerOn":
            updates["standby"] = True

        tasks = []
        futures: list[Any] = []
        app_id = None

        if updates["available"] and not updates["standby"]:
            updates["app"] = app = await self._get_active_app()

            if isinstance(app["app"], dict):
                app_id = app["app"].get("@id")

            if app_id and app_id[:7] != "tvinput":
                tasks.append("media")
                futures.append(self._get_media_state())

            if app_id == "tvinput.dtv":
                tasks.append("channel")
                futures.append(self._get_tv_active_channel())

        if full_update and updates["available"]:
            tasks.append("apps")
            futures.append(self._get_apps())

            if info.get("is-tv", "false") == "true":
                tasks.append("channels")
                futures.append(self._get_tv_channels())

        if len(tasks) > 0:
            results = await asyncio.gather(*futures)

            for (task, result) in zip(tasks, results):
                updates[task] = result

        if self._device is None:
            self._device = Device(updates)
        else:
            self._device.update_from_dict(updates)

        return self._device

    async def play_on_roku(
        self, video_url: str, params: dict[str, Any] | None = None
    ) -> None:
        """Play video via PlayOnRoku channel.

        Args:
            video_url: The URL to play on the Roku device.
            params: Dictionary of request parameters to send to the Roku device.
        """
        if params is None:
            params = {}

        request_params = {
            "t": "v",
            "u": video_url,
            **params,
        }

        encoded = urlencode(request_params)
        await self._request(f"input/15985?{encoded}", method="POST", encoded=True)

    async def launch(self, app_id: str, params: dict[str, Any] = None) -> None:
        """Launch an application on the Roku device.

        Args:
            app_id: The application ID to launch on the Roku device.
            params: Dictionary of request parameters to send to the Roku device.
        """
        if params is None:
            params = {}

        encoded = urlencode(params)
        await self._request(f"launch/{app_id}?{encoded}", method="POST", encoded=True)

    async def literal(self, text: str) -> None:
        """Send literal text to the Roku device.

        Args:
            text: The literal text to send to the Roku device.
        """
        for char in text:
            encoded = quote_plus(char)
            await self._request(f"keypress/Lit_{encoded}", method="POST")

    async def remote(self, key: str) -> None:
        """Emulate pressing a key on the remote.

        Args:
            key: The remote keypress to send to the Roku device.

        Raises:
            RokuError: Received an unexpected response from the Roku device.
        """
        key_lower = key.lower()
        if key_lower not in VALID_REMOTE_KEYS:
            raise RokuError(f"Remote key is invalid: {key}")

        if key_lower == "search":
            await self._request("search/browse", method="POST")
            return

        await self._request(f"keypress/{VALID_REMOTE_KEYS[key_lower]}", method="POST")

    async def search(self, keyword: str) -> None:
        """Emulate opening search and entering keyword on the Roku device.

        Args:
            keyword: The search keyword to send to the Roku device.
        """
        request_params = {
            "keyword": keyword,
        }

        await self._request("search/browse", method="POST", params=request_params)

    async def tune(self, channel: str) -> None:
        """Change the channel on Roku TV device.

        Args:
            channel: The channel number to send to the Roku device.
        """
        await self.launch("tvinput.dtv", {"ch": channel})

    async def _get_active_app(self) -> OrderedDict:
        """Retrieve active app for updates.

        Returns:
            An ordered Dictionary.

        Raises:
            RokuError: Received an unexpected response from the Roku device.
        """
        res = await self._request("/query/active-app")

        if not isinstance(res, dict) or "active-app" not in res:
            raise RokuError("Roku device returned a malformed result (active-app)")

        return res["active-app"]

    async def _get_apps(self) -> list[OrderedDict]:
        """Retrieve apps for updates.

        Returns:
            A list of ordered Python Dictionaries.

        Raises:
            RokuError: Received an unexpected response from the Roku device.
        """
        res = await self._request("/query/apps")

        if not isinstance(res, dict) or "apps" not in res:
            raise RokuError("Roku device returned a malformed result (apps)")

        if isinstance(res["apps"]["app"], OrderedDict):
            return [res["apps"]["app"]]

        return res["apps"]["app"]

    async def _get_device_info(self) -> OrderedDict:
        """Retrieve device info for updates.

        Returns:
            An ordered Dictionary.

        Raises:
            RokuError: Received an unexpected response from the Roku device.
        """
        res = await self._request("/query/device-info")

        if not isinstance(res, dict) or "device-info" not in res:
            raise RokuError("Roku device returned a malformed result (device-info)")

        return res["device-info"]

    async def _get_media_state(self) -> OrderedDict:
        """Retrieve media state for updates.

        Returns:
            An ordered Dictionary.

        Raises:
            RokuError: Received an unexpected response from the Roku device.
        """
        res = await self._request("/query/media-player")

        if not isinstance(res, dict) or "player" not in res:
            raise RokuError("Roku device returned a malformed result (player)")

        return res["player"]

    async def _get_tv_active_channel(self) -> OrderedDict:
        """Retrieve active TV channel for updates.

        Returns:
            An ordered Dictionary.

        Raises:
            RokuError: Received an unexpected response from the Roku device.
        """
        res = await self._request("/query/tv-active-channel")

        if not isinstance(res, dict) or "tv-channel" not in res:
            raise RokuError(
                "Roku device returned a malformed result (tv-active-channel)"
            )

        return res["tv-channel"]["channel"]

    async def _get_tv_channels(self) -> list[OrderedDict]:
        """Retrieve TV channels for updates.

        Returns:
            A list of ordered Python Dictionaries.

        Raises:
            RokuError: Received an unexpected response from the Roku device.
        """
        res = await self._request("/query/tv-channels")

        if not isinstance(res, dict) or "tv-channels" not in res:
            raise RokuError("Roku device returned a malformed result (tv-channels)")

        if res["tv-channels"] is None or "channel" not in res["tv-channels"]:
            return []

        if isinstance(res["tv-channels"]["channel"], OrderedDict):
            return [res["tv-channels"]["channel"]]

        return res["tv-channels"]["channel"]

    async def close_session(self) -> None:
        """Close open client session."""
        if self.session and self._close_session:
            await self.session.close()

    async def __aenter__(self) -> Roku:
        """Async enter.

        Returns:
            The Roku object.
        """
        return self

    async def __aexit__(self, *_exc_info: Any) -> None:
        """Async exit.

        Args:
            _exc_info: Exec type.
        """
        await self.close_session()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['rokuecp']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.0.0',
 'awesomeversion>=21.10.1',
 'backoff>=1.9.0',
 'cachetools>=4.2.4',
 'xmltodict>=0.12.0',
 'yarl>=1.6.0']

setup_kwargs = {
    'name': 'rokuecp',
    'version': '0.14.1',
    'description': 'Asynchronous Python client for Roku (ECP)',
    'long_description': '# Python: Roku (ECP) Client\n\nAsynchronous Python client for Roku devices using the [External Control Protocol](https://developer.roku.com/docs/developer-program/debugging/external-control-api.md).\n\n## About\n\nThis package allows you to monitor and control Roku devices.\n\n## Installation\n\n```bash\npip install rokuecp\n```\n\n## Usage\n\n```python\nimport asyncio\n\nfrom rokuecp import Roku\n\n\nasync def main():\n    """Show example of connecting to your Roku device."""\n    async with Roku("192.168.1.100") as roku:\n        print(roku)\n\n\nif __name__ == "__main__":\n    loop = asyncio.get_event_loop()\n    loop.run_until_complete(main())\n```\n\n## Setting up development environment\n\nThis Python project is fully managed using the [Poetry](https://python-poetry.org) dependency\nmanager. But also relies on the use of NodeJS for certain checks during\ndevelopment.\n\nYou need at least:\n\n- Python 3.9+\n- [Poetry](https://python-poetry.org/docs/#installation)\n- NodeJS 14+ (including NPM)\n\nTo install all packages, including all development requirements:\n\n```bash\nnpm install\npoetry install\n```\n\nAs this repository uses the [pre-commit](https://pre-commit.com/) framework, all changes\nare linted and tested with each commit. You can run all checks and tests\nmanually, using the following command:\n\n```bash\npoetry run pre-commit run --all-files\n```\n\nTo run just the Python tests:\n\n```bash\npoetry run pytest\n```\n',
    'author': 'Chris Talkington',
    'author_email': 'chris@talkingtontech.com',
    'maintainer': 'Chris Talkington',
    'maintainer_email': 'chris@talkingtontech.com',
    'url': 'https://github.com/ctalkington/python-rokuecp',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



"""Asynchronous Python client for Roku."""
from .exceptions import RokuConnectionError, RokuError  # noqa
from .models import Application, Channel, Device, Info, MediaState, State  # noqa
from .rokuecp import Roku  # noqa
